function love.load()
	setIdentity("")
	
	tree = {}
	tree.Type = "Root"
	
	y = 20
	x = 30
	
	skip_num = 0
	skips = 0
	
	scanFolder("", tree)
	
	mouse_y = 0
	highlighted_item = nil
	
	wh = love.graphics.getHeight()
	ww = love.graphics.getWidth()
end

function scanFolder(path, folder)
	local files = love.filesystem.enumerate(path)

	for i = 1, #files do
		local isFolder = love.filesystem.isDirectory(path .. "/" .. files[i])
		if isFolder then
			addFolder(files[i], path .. "/" .. files[i], folder)
		end
	end
	for i = 1, #files do
		local isFolder = love.filesystem.isDirectory(path .. "/" .. files[i])
		if not isFolder then
			addFile(files[i], path .. "/" .. files[i], folder)
		end
	end
end

function addFolder(name, path, parent)
	local item = {}
	item.Name = name
	item.Path = path
	item.Type = "Folder"
	
	table.insert(parent, item)
end

function addFile(name, path, parent)
	local item = {}
	item.Name = name
	item.Path = path
	item.Type = "File"
	
	table.insert(parent, item)
end

function setIdentity(to)
	love.filesystem.setIdentity('tmp')
	local new = ""
	local path = love.filesystem.getSaveDirectory()
	path = path:match("/(.+)") or path:match("%s:/(.+)")
	for s in path:gmatch("/") do new = new .. "../" end
	love.filesystem.setIdentity(new .. (to and to:gsub('^/', '') or ""))
end

function expandFolder(folder)
	if #folder > 0 then
		closeFolder(folder)
		return
	end

	scanFolder(folder.Path, folder)
end

function closeFolder(folder)
	for i = 1, #folder do
		table.remove(folder, 1)
	end
end

function love.mousepressed(X, Y, b)
	if b == "l" then
		if highlighted_item ~= nil then
			if highlighted_item.Type == "Folder" then
				expandFolder(highlighted_item)
			elseif highlighted_item.Type =="File" then
				os.execute('"' .. "C:" .. highlighted_item.Path .. '"')
			end
		end
	end
	if b == "wd" and y > wh then --Scroll down
		skip_num = skip_num + 1
	end
	if b == "wu" and skip_num > 0 then --Scroll up
		skip_num = skip_num - 1
	end
end

function love.keypressed(key)
	if key == "down" and y > wh then --Scroll down
		skip_num = skip_num + 1
	end
	if key == "up" and skip_num > 0 then --Scroll up
		skip_num = skip_num - 1
	end
end

function checkMouseOver(item)
	if mouse_y >= y - 2 and mouse_y < y + 15 then
		highlighted_item = item
		return true
	end
	return false
end

--Drawing Functions--

function drawFolder(folder, drawSelf)
	if drawSelf and skips > skip_num and y < wh then
		local mouseOver = checkMouseOver(folder)
		if mouseOver then
			love.graphics.rectangle("fill", 0, y - 2, ww, 17)
			love.graphics.setColor(0, 0, 0)
		end
		love.graphics.print(folder.Name .. "/", x, y)
		if mouseOver then
			love.graphics.setColor(255, 255, 255)
		end
		y = y + 17
	end
	skips = skips + 1
	x = x + 20
	for i = 1, #folder do
		drawItem(folder[i])
	end
	x = x - 20
end

function drawFile(file)
	if skips > skip_num and y < wh then
		local mouseOver = checkMouseOver(file)
		if mouseOver then
			love.graphics.rectangle("fill", 0, y - 2, ww, 17)
			love.graphics.setColor(0, 0, 0)
		end
		love.graphics.print(file.Name, x, y)
		if mouseOver then
			love.graphics.setColor(255, 255, 255)
		end
		y = y + 17
	end
	skips = skips + 1
end

function drawItem(item)
	if item.Type == "Folder" then
		drawFolder(item, true)
	end
	if item.Type == "File" then
		drawFile(item)
	end
	if item.Type == "Root" then
		drawFolder(item, false)
	end
end

function love.draw()
	mouse_y = love.mouse.getY()
	y = 2
	x = 30
	skips = 0

	drawItem(tree)
end